/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.generator.common;


/** To access all the AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;


/**
 * This class allows naviguation in the OpenCCM AST
 * and calls to map files.
 **/
public class GeneratorBase
     extends org.objectweb.corba.generator.common.Generator
  implements org.objectweb.corba.generator.common.GeneratorBaseOperations
{

    // ===========================================================
    //
    // Internal State.
    //
    // ===========================================================

    /**
     ** The Abstract Syntax Tree
     **/
    protected org.objectweb.corba.ast.api.Scope root_;

    /**
     ** Dependencies Tree of Declarations
     **/
    protected org.objectweb.ccm.util.Table dependencies_tree_;

    /**
     ** The target declaration to generate
     **/
    protected Declaration target_decl_;


    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param root - The file scope containing all declarations.
     */
    public GeneratorBase(org.objectweb.corba.ast.api.Scope root)
    {
        // Call the common Generator constructor
        super();

        // Init internal states
        root_ = root;
        dependencies_tree_ = new org.objectweb.ccm.util.Table();
        target_decl_ = null;

        //System.out.println("Abstract Syntax Tree : " + global_scope_);
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /**
     * Build dependencies tree for declarations passed in parameter
     *
     * @param decls - An array of declarations.
     **/
    private void
    dependenciesTreeUpdate(Declaration[] decls)
    {
        Declaration decl = null;
        org.objectweb.corba.ast.api.Scope parent = null;
        org.objectweb.ccm.util.Vector childs = null;

        for (int i=0;i<decls.length;i++)
        {
            decl  = decls[i];
            parent = decl.getParent();

            while (parent.getDeclKind() != DeclarationKind.dk_repository)
            {
                if (dependencies_tree_.containsKey(parent))
                {
                    childs = (org.objectweb.ccm.util.Vector)dependencies_tree_.get(parent);
                    if (childs.indexOf(decl)==-1)
                        childs.add(decl);
                    // continue;
                }
                else
                {
                    childs = new org.objectweb.ccm.util.Vector();
                    childs.add(decl);
                    dependencies_tree_.put(parent, childs);
                }

                decl = parent;
                parent = decl.getParent();
            }// end while
        }//end for

/*            System.err.println("### Dependency tree : ");
            for (java.util.Enumeration e = dependencies_tree_.keys() ; e.hasMoreElements() ;)
                System.err.println("    key "+((Declaration)e.nextElement()).getAbsoluteName());
*/
    }

    /**
     * Method called after generation.
     * Generally used to close files.
     **/
    private void
    after_generation()
    {
        // To redefine if necessary
    }

    // ===========================================================
    //
    // Public methods.
    //
    // ===========================================================

    /**
     * Is the declaration a business (i.e. a non system) declaration ?
     *
     * @param decl - The declaration.
     *
     * @return True if the declaration is business declaration, false otherwise.
     **/
    public boolean
    isBusinessDeclaration(Declaration decl)
    {
        String abs_name = decl.getAbsoluteName();
        if (abs_name.equals("::CORBA::TypeCode"))
        {
            //typecode_used_ = true;
            return false;
        }
        else if (abs_name.startsWith("::CORBA"))
        {
            //corba_used_ = true;
            return false;
        }
        else if (abs_name.startsWith("::Components"))
        {
            //components_used_ = true;
            return false;
        }

        return true;
    }


    /**
     * Visit a  declaration in the repository
     *
     * @param obj - The declaration to visit.
     **/
    public void
    visit(Declaration obj)
    {
        Declaration decl = null;

        // get the first non module scope that contains the target object
        Declaration nmodule = obj;
        Declaration current = obj;
        while ((current.getDeclKind()!= DeclarationKind.dk_repository) &&
               (current.getDeclKind()!= DeclarationKind.dk_module))
        {
            nmodule = current;
            current = nmodule.getParent();
        }

        // set the first non module scope as target declaration
//            System.err.println("### Generating for declaration : "+nmodule.getAbsoluteName());
        target_decl_ = nmodule;

        // get for dependencies.
        Declaration[] depend = nmodule.getDependencies();
//        Declaration[] depend = new Declaration[0];
/*                System.err.println("### Found Dependencies : "+depend.length);
                for (int i=0;i<depend.length;i++)
                    System.err.println("    "+depend[i].getAbsoluteName());
*/
        // build dependencies list in correct order
        //
        //  TODO : inter module cycles are not taken into account
        //         intra module cycles will be resolved by forward declarations
        //
        org.objectweb.ccm.util.Vector ordered_depend = new org.objectweb.ccm.util.Vector();
        for (int i=0;i<depend.length;i++)
        {
//                System.err.println("--+  : "+depend[i]);
//                System.err.println("--+  : "+depend[i].getId());
//                System.err.println("--+  : "+ordered_depend.indexOf(depend[i]));
            if ( isBusinessDeclaration(depend[i]) && (ordered_depend.indexOf(depend[i])==-1) )
                ordered_depend.add(depend[i]);
        }

        ordered_depend.add(nmodule);
        depend = (Declaration[]) ordered_depend.toArray(new Declaration[0]);

/*                System.err.println("### Dependency list cleaned : "+depend.length);
                for (int i=0;i<depend.length;i++)
                    System.err.println("    "+depend[i].getAbsoluteName());
*/
        java.util.Arrays.sort(depend, new org.objectweb.corba.ast.lib.DependencyComparator());

/*                System.err.println("### Dependency list sorted : "+depend.length);
                for (int i=0;i<depend.length;i++)
                    System.err.println("    "+depend[i].getAbsoluteName());
*/
        // build root dependencies
        org.objectweb.ccm.util.Vector root_dependencies = new org.objectweb.ccm.util.Vector();
        for (int i=0;i<depend.length;i++)
        {
            if (root_dependencies.indexOf(depend)==-1)
                root_dependencies.add(depend[i]);
        }

        // build dependencies tree (declaration tree)
        dependenciesTreeUpdate(depend);

        // visit root dependencies
/*                System.err.println("### Visiting root dependencies : " );//+ root_dependencies.size());
                for (int i=0;i<root_dependencies.size();i++)
                     System.err.println("    |    "+((Declaration)root_dependencies.get(i)).getAbsoluteName());
*/
            visit(root_dependencies);
    }

    /**
     * Visit declarations.
     *
     * @param vect - A vector containing Declarations to visit.
     **/
    public void
    visit(org.objectweb.ccm.util.Vector vect)
    {
        Declaration[] decls = (Declaration[]) vect.toArray(new Declaration[0]);
        for (int i=0;i<decls.length;i++)
        {
            // System.err.println("*********** visit : "+decls[i].getAbsoluteName());
/*            if (decls[i]==target_decl_)
            {
                // System.err.println("#### 1");
                visitContained(decls[i], "");
            }
            else if (decls[i].getDeclKind()==DeclarationKind.dk_module)
            {
                put("obj", decls[i]);
                put("vect", dependencies_tree_.get(decls[i]));
                // System.err.println("#### 2");
                map("MODULE_WITH_DEPENDENCIES");
            }
            else
            {
*/                visitContained(decls[i], "");
//            }
        }

    }

    /**
     * Visit a single contained declaration
     *
     * @param contained - The declaration to visit.
     * @param extension - used to map id
     **/
    public void
    visitContained(Declaration contained,
                   String extension)
    {
        String id = org.objectweb.corba.ast.lib.DeclarationKindImpl.toString(contained.getDeclKind()).toUpperCase()+ extension;

        put("obj", contained);
        // System.err.println("Visit contained :"+contained.getAbsoluteName()+" | "+id+" | "+contained.getPrefix() );
        map(id);
    }

    /**
     * Get a declaration.
     * name must be a valid declaration name.
     *
     * @param name - The name of the declaration to get.
     **/
    public Declaration
    getDeclaration(String name)
    throws org.objectweb.corba.generator.common.GenerationException
    {
        Declaration decl = null;

        if (root_ == null)
        {
            String msg = "[Error]: Interface Repository has not been fed!";
            throw new org.objectweb.corba.generator.common.GenerationException(msg);
        }

        // Search the object to generate
        decl = root_.lookup(name);
        if (decl==null)
        {
            String msg = "[Error]: object " + name + " not found : aborting generation !";
            throw new org.objectweb.corba.generator.common.GenerationException(msg);
        }
        return decl;
    }

    /**
     * Get contents of the root scope matching limited_types.
     *
     * @param limited_types - A logical combination of DeclarationKind.
     **/
    public org.objectweb.ccm.util.Vector
    getDeclarations(long limited_types)
    throws org.objectweb.corba.generator.common.GenerationException
    {
        Declaration[] decls = null;
        org.objectweb.ccm.util.Vector vect = new org.objectweb.ccm.util.Vector();

        if (root_ == null)
        {
            String msg = "[Error]: Interface Repository has not been fed!";
            throw new org.objectweb.corba.generator.common.GenerationException(msg);
        }

        decls = root_.getContents(true, limited_types);
        for (int i=0;i<decls.length;i++)
        {
            if( isBusinessDeclaration(decls[i]) )
            {
                // System.out.println("## find decl <"+decls[i].getAbsoluteName()+">");
                vect.add(decls[i]);
            }
        }
        return vect;
    }

}
