/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Demarey Christophe.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher;



/**
 * Launcher is used to run classes (call the "main" method).
 * Properties are read from a file.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class Launcher
{
    // ==================================================================
    //
    // Private methods.
    //
    // ==================================================================

    /**
     * Add jars to class loader.
     *
     * @param config - The configuration table.
     * @param loader - The classLoader to initialize.
     **/
    private static void
    updateClasspath(String[] jars, XBootClassLoader loader)
    {
        java.net.URL url = null;

        for (int i=0; i<jars.length; i++)
        {
            try {
                url = new java.net.URL(jars[i]);
            } catch(java.net.MalformedURLException exc) {
                System.err.println("[Error] : Malformed url " + url + "!");
            }
//            System.out.println("Loading : "+url);
            loader.addURL( url );
        }
    }

    /**
     * Compute Options for the class, i.e. :
     * - delete the last argument : the configuration file
     * - retrieve static options
     *
     * @param args - Dynamic options.
     * @param static_args - Static options.
     *
     * @return An array containing static and dynamic options.
     **/
    private static String[]
    computeOptions(String[] args, String[] static_args)
    {
        java.util.Vector options = new java.util.Vector();

        // Add dynamic options (whithout configuration file)
        for (int i=0; i<args.length-1; i++)
        {
            options.add(args[i]);
        }

        // Add static options
        for (int i=0; i<static_args.length; i++)
        {
            java.util.StringTokenizer st = new java.util.StringTokenizer(static_args[i]);

            while (st.hasMoreTokens())
            {
                 options.add( st.nextToken() );
            }
        }

        return (String[])options.toArray(new String[0]);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     **/
    public static void
    main(String[] args)
    {
        boolean ok = true;
        XBootClassLoader loader = new XBootClassLoader();
        ConfigDecoder config = new ConfigDecoderImpl();
        String[] classargs = null;
        String configurationFile = null;

        if (args.length < 1)
        {
            System.err.println("Bad usage !");
            System.err.println("java Launcher [options] xml_file");
            System.err.println("where xml_file is the configuration file.");
            System.exit(0);
        }

        // Get the last argument
        configurationFile = args[args.length-1];

        // Parse the xml configuration file
        ok = config.parse(configurationFile);
        if (!ok)
        {
            System.err.println("[Launcher] : Problem when parsing " + configurationFile);
            System.exit(0);
        }

        // Init class loader
        updateClasspath( config.getJarList(), loader );
//        System.out.println("nom de la classe = "+config.getClassName());

        // Compute options
        classargs = computeOptions(args, config.getStaticoptionList());
//        for (int i=0; i<classargs.length; i++)
//            System.out.println("Option "+i+" : "+classargs[i]);

        // Init class to launch
        Class[] params = { args.getClass() };
        Object[] in_params = { classargs };
        String classname = config.getClassName();

        try{
            java.lang.reflect.Method method = Class.forName(classname, false, loader).getMethod("main", params);
            method.invoke(null, in_params);
        }catch(Exception e){
            System.err.println("[Launcher] Error : ");
            e.printStackTrace();
        }
    }
}
