/*===========================================================================

ObjectWeb Util Launcher package.
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): Jerome Moroy.

===========================================================================*/

package org.objectweb.util.launcher;


/** To use the File class */
import java.io.File;

/** To use the List class */
import java.util.List;

/** To use the Iterator class */
import java.util.Iterator;

/** To use the Vector class */
import java.util.Vector;


/**
 * Parses a Java object wihch representes an XML Configuration.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * @version 0.1
 */
public class ConfigDecoderImpl
  implements ConfigDecoder
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The class name */
    private String className_;

    /** The jars list */
    private Vector jars_;

    /** The static options list */
    private Vector staticOptions_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Empty constructor
     */
    public ConfigDecoderImpl()
    {
        className_ = null;
        jars_ = new Vector();
        staticOptions_ = new Vector();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Returns a CONFIGURATION Object corresponding to the contents of
     * the file containing in the INCLUDE tag.
     *
     * @param include - The INCLUDE tag
     *
     * @return The associated CONFIGURATION, null if an error occurred.
     */
    private CONFIGURATION
    getConfiguration(INCLUDE include)
    {
        try{
            return CONFIGURATIONImpl.unmarshal( new File(include.getFile()) );
        }catch(java.lang.Exception e){
            e.printStackTrace();
            return null;
        }
    }

    /**
     * Parses a configuration.
     *
     * @param conf - The Configuration to parse.
     */
    private void
    parseConfiguration(CONFIGURATION config)
    {
        if (config != null)
        {
            List elements = null;

            elements = config.getELEMList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                ELEM e = (ELEM)i.next();
                if (e.getName().compareTo("classname") == 0)
                    parseClassnameElement(e);
                if (e.getName().compareTo("jar") == 0)
                    parseJarElement(e);
                if (e.getName().compareTo("staticoption") == 0)
                    parseStaticoptionElement(e);
            }
        }
    }

    /**
     * Parses a "classname" element.
     *
     * @param elem - The Classname Element to parse.
     */
    private void
    parseClassnameElement(ELEM elem)
    {
        ATOM atom = null;

        atom = elem.getATOM();
        if (atom != null)
            className_ = atom.getXmlclass();
    }

    /**
     * Parses a "jar" element.
     *
     * @param elem - The Jar Element to parse.
     */
    private void
    parseJarElement(ELEM elem)
    {
        CONFIGURATION conf = null;
        INCLUDE include = null;

        conf = elem.getCONFIGURATION();
        if (conf != null)
        {
            parseJarConfiguration(conf);
            return;
        }

        include = elem.getINCLUDE();
        if (include != null)
        {
            conf = getConfiguration(include);
            parseConfiguration(conf);
            return;
        }

    }

    /**
     * Parses a configuration contained in a "jar" element.
     *
     * @param conf - The Jar Configuration to parse.
     */
    private void
    parseJarConfiguration(CONFIGURATION conf)
    {
        if (conf != null)
        {
            List elements = null;

            elements = conf.getELEMList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                ELEM e = (ELEM)i.next();
                jars_.add( e.getName() );
            }
        }
    }

    /**
     * Parses a "staticoption" element.
     *
     * @param elem - The Staticoption Element to parse.
     */
    private void
    parseStaticoptionElement(ELEM elem)
    {
        CONFIGURATION conf = null;
        INCLUDE include = null;

        conf = elem.getCONFIGURATION();
        if (conf != null)
        {
            parseStaticoptionConfiguration(conf);
            return;
        }

        include = elem.getINCLUDE();
        if (include != null)
        {
            conf = getConfiguration(include);
            parseConfiguration(conf);
            return;
        }

    }

    /**
     * Parses a configuration contained in a "staticoption" element.
     *
     * @param conf - The Staticoption Configuration to parse.
     */
    private void
    parseStaticoptionConfiguration(CONFIGURATION conf)
    {
        if (conf != null)
        {
            List elements = null;

            elements = conf.getELEMList();
            for(Iterator i = elements.iterator() ; i.hasNext() ; )
            {
                ELEM e = (ELEM)i.next();
                staticOptions_.add( e.getName() );
            }
        }
    }


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Parses the properties file.
     *
     * @param file The property file containing the configuration.
     *
     * @return The status (true if ok, else false).
     */
    public boolean
    parse(String file)
    {
        CONFIGURATION config;

        if(file != null){
            try{
                config = CONFIGURATIONUnmarshaller.unmarshal(new File(file));
            }catch (java.io.IOException e){
                System.err.print("[I/O Error] : ");
                e.printStackTrace();
                return false;
            }
        }
        else
        {
            System.err.println("The file parameter is null ! ");
            return false;
        }

        parseConfiguration(config);
        return true;
    }

    /**
     * Get the class to launch.
     *
     * @return The class name.
     */
    public String
    getClassName()
    {
        return className_;
    }

    /**
     * Get the jar list associated to the CONFIGURATION.
     *
     * @return An array of jars.
     */
    public String[]
    getJarList()
    {
        return (String[])jars_.toArray(new String[0]);
    }

    /**
     * Get the static option list associated to the CONFIGURATION.
     *
     * @return An array of static options.
     */
    public String[]
    getStaticoptionList()
    {
        return (String[])staticOptions_.toArray(new String[0]);
    }

}
